/*
**   WavePlayer
**
**	v1.0 LDJ 7/30/06  
*/

#include "processor.h"
#include "sdmmc.h"
#include "fileio.h"
#include "../Audio/Audio PWM.h"
#include "../Wave/Wave.h"
#include "../Delay/Delay.h"
#include "../buttons/buttons.h"
#include "../random/rand.h"
#include "../LCD/LCD.h"
#include "../8 COMM3/conU2.h"

_FOSCSEL( FNOSC_PRIPLL )
_FOSC( FCKSM_CSDCMD & OSCIOFNC_ON & POSCMD_XT )
_FWDT( FWDTEN_OFF )
_FPOR( PWRTEN_ON & FPWRT_PWR128 )

#define LIST_MAX    40
char list[LIST_MAX][8];        // list of file names matching
char name[13];
char s[16];

int comp( const void *a, const void *b)
{ // inverse alphabetical order
    return -memcmp( a, b, 8); 
} // comp


unsigned play( int i)
{
    // form a complete file name
    strncpy( name, list[i], 8); name[8] = '\0';
    strcat( name, ".WAV");
    // update display
    setLCDC( 0x40);         // on the second line
    putsLCD( name);
    // play the file, terminate on error or key pressed
    return  playWAV( name);

} // play

main( void)
{
    int i, n, c;
    int Index = 0;
    
    int key;

	
	// dsPIC - Setup PLL for 40 MIPS with 8MHz external crystal
    CLKDIV = 0;		// 0x0040 for 16 MIPS;
    PLLFBD = 40;	// 30 for 16 MIPS
    
    TRISA = 0xff80;
    _TRISD6 = 1;
    _TRISD7 = 1;
    _TRISA7 = 1;
    _TRISD13 = 1;
    // dsPIC - need to setup ADC pins
    AD1PCFGH = 0xFFFF;		// all ADC inputs are digital
    AD1PCFGL =0x0FFDF;		// AN5 analog for potentiomenter input
    AD2PCFGL = 0xFFFF;     	// all ADC inputs are digital
       
    initSD();
    initLCD();
            
	//    initU2();
    // randomize
    //getKEY();
    randomize( TMR1);
	
	while(1)	// main loop
	{
    // repeatedly try to mount the storage device
    while ( !mount())
 	{
    	// wait for card to be inserted 
    	while( !detectSD());
    	// and contact debounce
    	Delayms( 100);
    } 	
    
	    while( 1) // main loop (card mounted)
	    {
	        if ( !detectSD())
	        {
	        	PORTA = 0;
	        	clrLCD();
	        	break;
	        	
	        }
	    
		    // successfully mounted
		    PORTA = 0xFF;
		    clrLCD(); homeLCD();
	
	        // create a list of all WAV files on the root
	        n = listTYPE( list, LIST_MAX, "WAV");
	
	        // show how many songs on the first line
	        sprintf( s, "%d WAV files  ", n);
	        putsLCD( s);
	
	        if ( n < 1)
	            while( 1);      // no WAV files found keep looping
	
	        // sort the list
	        qsort( list, n, 8, comp);
	
	
	        // wait for a command, play prev., play next, seq., shuffle
	        c = getKEY();
	        if (( c & 2) || ( c & 1))
	        {
	        
	            // play all WAV files sequential or shuffle    
	            while(( n>0) && (!readKEY()))
	            {
	                clrLCD(); homeLCD();
	
	                // select next file in shuffle mode or sequential mode
	                if ( c & 1)    // shuffle 
	                {
	                    i = rand() % n;
	                    sprintf(s, "shf %d", i);
	                    putsLCD( s);
	                }
	                else            // sequential
	                {
	                    i = n-1;
	                    sprintf(s, "seq %d", i);
	                    putsLCD( s);
	                }
	
	                // form a complete file name
	                if ( !play( i))
	                    break;  
	
	                // remove file name from the list
	                if ( i < n-1)
	                    memcpy( list[i], list[n-1], 8);
	    
	                // remove last file from the list
	                n--;
	                    
	            } // play all WAV files
	
	        } // sequential or shuffle
	
	       if ( c & 8)  // previous file single
	       { 
	            if ( --Index <0)    
	                Index = n-1;
	            play( Index);
	
	        } // previous
	
	        if ( c & 4) // current file
	        {            
	            if ( Index >= n)
	                Index = n-1;
	            // form a complete file name
	
	            // play the file, terminate on error or key pressed
	            play( Index++);
	        }   // next 
	
	    } // main loop (card mounted)
    } // main loop
} //main 
